from collections import deque

def _pref_prioridade(pref):
    return {
        a: {b: idx for idx, b in enumerate(a_pref)}
        for a, a_pref in pref.items()
    }

def match(*, A, B, A_pref, B_pref):

    B_prioridade = _pref_prioridade(B_pref)
    lista_espera = {a: deque(bs) for a, bs in A_pref.items()}
    par = {}
    
    restante_A = set(A)
    while len(restante_A) > 0:
        a = restante_A.pop()
        b = lista_espera[a].popleft()
        if b not in par:
            par[b] = a
        else:
            a0 = par[b]
            b_prefer_a0 = B_prioridade[b][a0] < B_prioridade[b][a]
            if b_prefer_a0:
                restante_A.add(a)
            else:
                restante_A.add(a0)
                par[b] = a
    
    return [(a, b) for b, a in par.items()]


from gale_shapley import algorithm

def test_match_comerciante_influenciador():
    comerciantes = {'Cunha SA', 'Vieira SA', 'Lineu SA', 'Loubach SA'}
    influenciadores = {'Gildárcio', 'Shigemura', 'Henrique', 'Jean'}

    comerciante_influenciador_pref = {
        'Cunha SA': ['Gildárcio', 'Shigemura', 'Henrique', 'Jean'], 
        'Vieira SA': ['Jean', 'Shigemura', 'Gildárcio', 'Henrique'], 
        'Lineu SA':['Shigemura', 'Henrique', 'Gildárcio', 'Jean'], 
        'Loubach SA':['Shigemura', 'Henrique', 'Gildárcio', 'Jean']
    }
    influenciador_comerciante_pref = {
        'Gildárcio': ['Cunha SA', 'Vieira SA', 'Lineu SA', 'Loubach SA'], 
        'Shigemura': ['Loubach SA', 'Lineu SA', 'Cunha SA', 'Vieira SA'], 
        'Henrique': ['Cunha SA', 'Loubach SA', 'Vieira SA', 'Lineu SA'], 
        'Jean': ['Vieira SA', 'Cunha SA', 'Loubach SA', 'Lineu SA']
    }

    pareamento = algorithm.match(A = comerciantes, B = influenciadores, A_pref = comerciante_influenciador_pref, B_pref = influenciador_comerciante_pref)
    resultado_esperado = [('Loubach SA', 'Shigemura'), ('Cunha SA', 'Gildárcio'), ('Vieira SA', 'Jean'), ('Lineu SA', 'Henrique')]
    print(F'Resultado {pareamento} não foi igual ao esperado {resultado_esperado}')

    for resultado in resultado_esperado:
        assert resultado in pareamento

def test_match_produto_influenciador():
    influenciadores = {'Gildárcio', 'Shigemura', 'Henrique', 'Jean'}
    produtos = {'Armário', 'Bolsa', 'Camarão', 'Dentadura'}

    produto_influenciador_pref = {
        'Armário': ['Gildárcio', 'Shigemura', 'Henrique', 'Jean'], 
        'Bolsa': ['Jean', 'Shigemura', 'Gildárcio', 'Henrique'], 
        'Camarão':['Shigemura', 'Henrique', 'Gildárcio', 'Jean'], 
        'Dentadura':['Shigemura', 'Henrique', 'Gildárcio', 'Jean']

    }
    influenciador_produto_pref = {
        'Gildárcio': ['Armário', 'Bolsa', 'Camarão', 'Dentadura'], 
        'Shigemura': ['Camarão', 'Armário', 'Bolsa', 'Dentadura'], 
        'Henrique': ['Armário', 'Dentadura', 'Camarão', 'Bolsa'], 
        'Jean': ['Bolsa', 'Camarão', 'Dentadura', 'Armário']    
    }

    pareamento = algorithm.match(A = produtos, B = influenciadores, A_pref = produto_influenciador_pref, B_pref = influenciador_produto_pref)
    resultado_esperado = [('Armário', 'Gildárcio'), ('Camarão', 'Shigemura'), ('Bolsa', 'Jean'), ('Dentadura', 'Henrique')]
    print(F'Resultado {pareamento} não foi igual ao esperado {resultado_esperado}')

    for resultado in resultado_esperado:
        assert resultado in pareamento


from setuptools import find_packages, setup
import pathlib

here=pathlib.Path(__file__).parent.resolve()
long_description=(here / 'README.md').read_text(encoding='utf-8')

setup(
    name='galeshapley',
    packages=find_packages(),
    version='0.0.1',
    description='Gale Shapley Algorithm',
    long_description=long_description,
    url='https://gitlab.com/stamps-group/galeshapley',
    author='Gomes, J.M.',
    license='MIT',
    install_requires=[],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==4.4.1'],
    test_suite='tests',
)